/*
 *		q s o r t _ c
 *
 *	The qsort function sorts an array of n objects, the initial
 *	element of which is pointed to by 'base'.  The size of each
 *	object is specified by 'width'.
 *
 *	The contents of this array are sorted into ascending order
 *	according to a user supplied comparison funcion '_cmpfunction'
 *	which is called with two arguments that point to the objects
 *	being compared.  The function must return an integer less than,
 *	equal to, or greater than zero if the first argument is 
 *	considered to be repectively less than, equal to, or greater
 *	than the second.
 *
 *	If two elements conmpare as equal then their order in the
 *	array is unspecified.
 *
 *	This routine is written to use a QUICKSORT approach
 *
 * adjusted 4 February 1995 by Joachim Van der Auwera to be part of the
 * PROGS SMS C library
 */

#include "mem_h"
#include "array_h"

Error ARRAYSort(void *base, Size count, Size width,
				int (*compare)(const void *, const void *))
{
	/* sort (char base[width])[count] using quicksort */
	while (1 < count)
	{
		/* worth sorting */
		Size i = 0;
		Size j = count - 1;
		char *qi = base;
		char *qj = qi + width * j;
		char *qp = qj;

		while (i < j)
		{
			/* partition about pivot */
			while (i < j && (*compare)(qi, qp) <= 0)
			{
				++i;
				qi += width;
			}
			while (i < j && (*compare)(qp, qj) <= 0)
			{
				--j;
				qj -= width;
			}
			if (i < j)
			{
				MEMSwap(width,qi,qj);
				++i;
				qi += width;
			}
		}
		if (qi != qp)
		{
			MEMSwap(width,qi,qp);
		}
		j = count - i;
		if (j < i)
		{
			/* recurse on smaller partition */
			if (1 < j)
				ARRAYSort(qi, j, width, compare);
			count = i;
		}
		else
		{
			/* lower partition is smaller */
			if (1 < i)
				ARRAYSort(base, i, width, compare);
			base = (void *)qi;
			count = j;
		}
	}
	return ERR_OK;
}
