/*
	Standard library file for PROGS SMS C library

	job handling library
	JOBLoad

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, started 15 December 1994
	01.04.1996 allocated dataspace always made even, needed for Minerva
*/

#include "io_h"
#include "str_h"
#include "job_h"

typedef struct {
	Size length;
	void *start;
	Job owner;
	char *released_flag;
	short tag, priority_acc, wait, priority;
	char priority_absolute, priority_increment;
	Job waiting;
	void *exception_vector, *save_area;
	long  d0, d1, d2, d3, d4, d5, d6, d7;
	char *a0,*a1,*a2,*a3,*a4,*a5,*a6,*a7;
	short sr;
	long pc;
	char relative, spare;
	} JobHeader;

Error JOBLoad(char *name, char *path, Priority priority, Job owner, char wait,
		int stringcount, char **strings, int channelcount, Channel *channels,
		Job *jobid)
{
	Error err;
	Size length, data, para;
	Channel chid;
	FileType ft;
	char *base;
	Job job;
	int i;
	JobHeader *jh;					/* job header */
	char *parapos;					/* current position in area for parameter */

	/* open the file and test for the filetype */
	err=IOOpenPath(name,OPEN_READ,path,&chid);
	if (err) return err;
	err=IOFileTypeGet(chid,&ft);
	if (err) return err;
	if (ft!=FILETYPE_EXECUTABLE) return ERR_ISYN;

	/* calculate amount of memory needed for parameters */
	para= stringcount + channelcount*sizeof(Channel) + 10;
	/* add the length needed for of all the strings */
	for (i=0; i<stringcount; i++)
	{
		Size len; char *str, havespace=FALSE;
		len=strlen(strings[i]);
		para+=len;
		for (str=strings[i];len;len--,str++)
		{
			if (*str=='\"') para++;
			if (*str==' ' && !havespace) havespace=TRUE, para+=2;
		}
	}

	/* allocate memory for the module */
	err=IOLength(chid,&length);
	if (err) { IOClose(chid); return err; }
	err=IODataSpaceGet(chid,&data);
	if (err) { IOClose(chid); return err; }
	if (data<0) data=0; 			/* make sure dataspace is positive */
	para=(para+1)&-2;				/* make para even - needed for Minerva */
	data=(data+1)&-2;				/* make data even - needed for Minerva */
	err=JOBCreate(owner, length, data+para+2, NULL, &job ,&base);
	if (err) { IOClose(chid); return err; }
	if (jobid) *jobid=job;

	/* adjust the registers for the system */
	jh=(JobHeader *)(base-sizeof(JobHeader));
	jh->a7-=para;
	parapos=jh->a7; 				/* start at the beginning */

	/* load the module */
	err=IOFileLoad(chid,length,base);
	IOClose(chid);
	if (err)
	{
		JOBRemove(job, ERR_IMEM);
		return err;
	}

	/* now pass the parameters */
	{
		/* fill in the channel table */
		Channel *list;
		short *count=(short *)parapos;
		*count++=channelcount;
		list=(Channel *)count;
		for (i=channelcount;i;i--)
		{
			*list++=*channels++;
		}
		parapos=(char *)list;
	}
	if (stringcount)
	{
		short *slen=(short *)parapos;
		char *string=parapos+sizeof(short);
		for (i=0; i<stringcount; i++)
		{
			Size len; char *str, havespace=(*strings[i]=='\"');
			for (str=strings[i],len=strlen(strings[i]);
				 len && !havespace;
				 len--,str++)
			{
				if (*str==' ') havespace=TRUE;
			}
			if (havespace) *string++='\"';
			for (str=strings[i],len=strlen(strings[i]);len;len--)
			{
				*string++=*str;
				if (*str++=='\"' && havespace) *string++='\"';
			}
			if (havespace) *string++='\"';
			*string++=' ';
		}
		*slen=string-parapos-3;
	}
	else
		*(short *)parapos=0;

	/* and activate the job */
	if (priority)
	{
		if (wait)
			return JOBActivateWait(job, priority);
		else
			return JOBActivate(job, priority);
	}
	else
		return ERR_OK;
}
