/*
	Standard library file for PROGS SMS C library

	thing handling library
	THINGExec

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera,
	started 05 January 1995, based on JOBLoad_c
	01.04.1996 allocated dataspace always made even, needed for Minerva
*/

#include "io_h"
#include "str_h"
#include "thing_h"

typedef struct {
	Size length;
	void *start;
	Job owner;
	char *released_flag;
	short tag, priority_acc, wait, priority;
	char priority_absolute, priority_increment;
	Job waiting;
	void *exception_vector, *save_area;
	long  d0, d1, d2, d3, d4, d5, d6, d7;
	char *a0,*a1,*a2,*a3,*a4,*a5,*a6,*a7;
	short sr;
	long pc;
	char relative, spare;
} JobHeader;

struct _Thing {
	int flag,type;
	int headeroffset;
	Size headerlength;
	Size dataspace;
	int startoffset;
};

#define EXECTHING_FLAG 0x54484725
#define EXECTHING_TYPE 1

Error THINGExec(char *name, Priority priority, Job owner, char wait,
		int stringcount, char **strings, int channelcount, Channel *channels,
		Job *jobid)
{
	Error err;
	Size length, data, para;
	Channel chid;
	char *base;
	Job job;
	int i;
	JobHeader *jh;					/* job header */
	char *parapos;					/* current position in area for parameter */
	Thing thing;

	/* start by using the thing - and check that it is really executable */
	err=THINGUse(name,0,&thing,NULL);
	if (err) return err;
	if (thing->flag!=EXECTHING_FLAG || thing->type!=EXECTHING_TYPE)
	{
		err=ERR_ISYN;
		THINGFree(name);
		return err;
	}

	/* calculate amount of memory needed for parameters */
	para= stringcount + channelcount*sizeof(Channel) + 10;
	/* add the length needed for all the strings */
	for (i=0; i<stringcount; i++)
	{
		Size len; char *str, havespace=FALSE;
		len=strlen(strings[i]);
		para+=len;
		for (str=strings[i];len;len--,str++)
		{
			if (*str=='\"') para++;
			if (*str==' ' && !havespace) havespace=TRUE, para+=2;
		}
	}

	/* allocate memory for the module */
	para=(para+1)&-2;				/* make para even - needed for Minerva */
	err=JOBCreate(owner, thing->headerlength, (thing->dataspace+para+2)&(-2),
				  thing->startoffset ? (char *)thing+thing->startoffset : NULL,
				  &job ,&base);
	if (err) { THINGFree(name); return err; }
	if (jobid) *jobid=job;

	/* the thing has to be used by the job which is created */
	THINGFree(name);
	err=THINGUseFor(name,0,job,FOREVER,&thing,NULL);
	if (err) { JOBRemove(job,err); return err; }

	/* adjust the registers for the system */
	jh=(JobHeader *)(base-sizeof(JobHeader));
	jh->a7-=para;
	parapos=jh->a7; 				/* start at the beginning */

	/* have to copy the job header */
	MEMCopy(thing->headerlength,(char *)thing+thing->headeroffset,base);

	/* now pass the parameters */
	{
		/* fill in the channel table */
		Channel *list;
		short *count=(short *)parapos;
		*count++=channelcount;
		list=(Channel *)count;
		for (i=channelcount;i;i--)
		{
			*list++=*channels++;
		}
		parapos=(char *)list;
	}
	if (stringcount)
	{
		short *slen=(short *)parapos;
		char *string=parapos+sizeof(short);
		for (i=0; i<stringcount; i++)
		{
			Size len; char *str, havespace=(*strings[i]=='\"');
			for (str=strings[i],len=strlen(strings[i]);
				 len && !havespace;
				 len--,str++)
			{
				if (*str==' ') havespace=TRUE;
			}
			if (havespace) *string++='\"';
			for (str=strings[i],len=strlen(strings[i]);len;len--)
			{
				*string++=*str;
				if (*str++=='\"' && havespace) *string++='\"';
			}
			if (havespace) *string++='\"';
			*string++=' ';
		}
		*slen=string-parapos-3;
	}
	else
		*(short *)parapos=0;

	/* and activate the job */
	if (priority)
	{
		if (wait)
			return JOBActivateWait(job, priority);
		else
			return JOBActivate(job, priority);
	}
	else
		return ERR_OK;
}
