; WIN library for the PROGS SMS C library
;   WINPointerRead      read pointer
; written by Joachim Van der Auwera for PROGS
; (c) 1994 by PROGS, Professional & Graphical Software, all rights reserved
; started 1 December 1994,
; 14.04.1995 adjusted to handle secondary window move/scale events
; 24.05.1996 window move/scale request in secondary window now handled
;            properly - right sprite, right window which scans the event
; 25.05.1996 when out of window in secondary during window request -> no
;            timeout possible (makes window move/scale work better in ProWesS)
        section text
        include win1_inc_qdos_io
        include win1_inc_qdos_sms
        include win1_inc_qdos_pt
        include win1_inc_keys_err
        include win1_inc_keys_sys
        include win1_inc_ptr_keys
        include win1_inc_aid_mac
chn_tag equ     $10             ; missing, tag in channel definition block
;
; read pointer
window  equ     4               ; window channel id
time    equ     8               ; timeout
term    equ     10              ; terminate
xpos    equ     14              ; *xpos
ypos    equ     18              ; *ypos
prec    equ     22              ; pointer record
RSP     equ     24
_WINPointerRead xsub
        regsave d2/d3/a2/a3/a4/a5
        move.l  RSP+window(sp),a1
        move.l  RSP+term(sp),d3

        ; test for possible window request in secondary window
        btst    #7,d3           ; window request event ??
        beq.s   read            ; no -> no problem

        ; figure out whether this window is a primary
        ; normally I should use iow.xtop to do this
        ; however, the window has to be fully visible for iow.xtop to work,
        ; therefore, I use some tricks here

        trap    #0              ; make this safe !!

        moveq   #sms.info,d0
        trap    #do.sms2

        move.l  a1,d0           ; calculate offset in channel table
        add.w   d0,d0
        add.w   d0,d0
        move.l  sys_chtb(a0),a2 ; base of channel table
        tst.l   0(a2,d0.w)      ; assure that channel is valid
        bmi.s   read
        move.l  0(a2,d0.w),a2   ; base of channel definition block
        lea     sd.extnl(a2),a2 ; make that base of extended ...

        btst    #7,sd_prwin(a2) ; is it a primary ??
        beq.s   secondary       ; yes -> just ask for window request event

        andi.w  #$dfff,sr       ; back to user mode

read    move.l  a1,a0           ; window channel id
        move.l  d3,d2           ; window request event
        move.w  RSP+time(sp),d3 ; timeout
        move.l  RSP+xpos(sp),a1
        move.w  (a1),d1
        swap    d1
        move.l  RSP+ypos(sp),a1
        move.w  (a1),d1
        move.l  RSP+prec(sp),a1
        moveq   #iop.rptr,d0
        trap    #3

        move.l  RSP+ypos(sp),a1
        move.w  d1,(a1)
        swap    d1
        move.l  RSP+xpos(sp),a1
        move.w  d1,(a1)
        subend

null    dc.l    0
sw_move dc.l    null
        dc.l    0,0,0,0
        dc.l    sp4_move
sw_size dc.l    null
        dc.l    0,0,0,0
        dc.l    sp4_size

secondary
        ; window request in a secondary window
        ; unfortunately, this is not handled properly by the OS, so ... hack
        ; three things have to be done :
        ; - redirect the call to the primary window
        ; - change the pointer sprite
        ; - just ask for a keystroke event

        ; redirect the call to the primary window
        move.l  sd_pprwn(a2),d0 ; channel definition block of primary
        move.l  sys_chtb(a0),a1 ; start of channel table
scan    cmp.l   (a1)+,d0        ; scan table for channel block
        bne.s   scan
        subq.l  #4,a1           ; compensate for post increment
        sub.l   sys_chtb(a0),a1 ; offset in table
        exg     d0,a1           ; exchange
        move.w  chn_tag(a1),d1  ; tag for channel id
        swap    d1              ; has to be in high word
        lsr.l   #2,d0           ; offset should just be a counter
        move.w  d0,d1           ; in low word of channel id
        lea     sd.extnl(a1),a3 ; make that base of extended ...
        move.l  d1,a1           ; put channel id where expected

        ; change the pointer sprite
        move.l  sd_wwdef(a3),a4 ; keep old value
        move.l  sd_sewll(a3),a5 ; store secondary window link list
        clr.l   sd_sewll(a3)
        lea     sw_move(pc),a2
        btst    #1,d3           ; window size ?
        beq.s   nosize
        lea     sw_size(pc),a2
nosize  move.l  a2,sd_wwdef(a3)

        andi.w  #$dfff,sr       ; back to user mode

        move.l  a1,a0           ; window channel id
        move.l  RSP+prec(sp),a1
handle  moveq   #17,d2          ; wait for keystroke or out of window
        move.w  RSP+time(sp),d3 ; timeout
        moveq   #iop.rptr,d0
        trap    #3

        btst    #4,pp_pevnt(a1)
        beq.s   inwin

        ; wait for pointer to be in window again
        moveq   #-1,d3          ; timeout forever
        moveq   #32,d2          ; wait for in window
        moveq   #iop.rptr,d0
        trap    #3
        bra.s   handle          ; back in window -> try again

inwin   move.l  a4,sd_wwdef(a3) ; restore sprite and subwindow definition
        move.l  a5,sd_sewll(a3) ; restore secondary window link list

        move.l  RSP+ypos(sp),a1
        move.w  d1,(a1)
        swap    d1
        move.l  RSP+xpos(sp),a1
        move.w  d1,(a1)
        subend
;
; the sprite while moving are a bit different from when drawn, so define here
* Sprite move
*
*       Mode 4
*       +-------|--------+
*       |aaaaaaaaaaa     |
*       |aawwwwwwwwa     |
*       |aawwrrrrwwa     |
*       |aawwrrrrwwaaaaaa|
*       -aawwrrwwwwwwwwaa-
*       |aawwwwwwwwrrwwaa|
*       |aaaaaawwrrrrwwaa|
*       |    aawwrrrrwwaa|
*       |    aawwwwwwwwaa|
*       |    aaaaaaaaaaaa|
*       +-------|--------+
*
sp4_move
        dc.w    $0100,$0000
        dc.w    16,10,7,4
        dc.l    sc4_move-*
        dc.l    sm4_move-*
        dc.l    0
sc4_move
        dc.w    $0000,$0000
        dc.w    $3F3F,$C0C0
        dc.w    $303F,$C0C0
        dc.w    $303F,$C0C0
        dc.w    $333F,$FCFC
        dc.w    $3F3F,$CCFC
        dc.w    $0303,$0CFC
        dc.w    $0303,$0CFC
        dc.w    $0303,$FCFC
        dc.w    $0000,$0000
        dc.w    $FFFF,$E0E0
        dc.w    $FFFF,$E0E0
        dc.w    $FFFF,$E0E0
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $0F0F,$FFFF
        dc.w    $0F0F,$FFFF
        dc.w    $0F0F,$FFFF
sm4_move
        dc.w    $FFFF,$E0E0
        dc.w    $FFFF,$E0E0
        dc.w    $FFFF,$E0E0
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $0F0F,$FFFF
        dc.w    $0F0F,$FFFF
        dc.w    $0F0F,$FFFF
        dc.w    $0F0F,$FCFC
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
* Sprite size
*
*       Mode 4
*       +-------|--------+
*       |aaaaaaaaaaaaaaaa|
*       |aawwwwwwwwwwwwaa|
*       |aawwrrrrrrrrwwaa|
*       |aawwrrrrrrrrwwaa|
*       -aawwrrwwwwwwwwaa-
*       |aawwrrwwrrrrwwaa|
*       |aawwrrwwrrrrwwaa|
*       |aawwwwwwwwwwwwaa|
*       |aaaaaaaaaaaaaaaa|
*       +-------|--------+
*
sp4_size
        dc.w    $0100,$0000
        dc.w    16,9,7,4
        dc.l    sc4_size-*
        dc.l    sm4_size-*
        dc.l    0
sc4_size
        dc.w    $0000,$0000
        dc.w    $3F3F,$FCFC
        dc.w    $303F,$0CFC
        dc.w    $303F,$0CFC
        dc.w    $333F,$FCFC
        dc.w    $333F,$0CFC
        dc.w    $333F,$0CFC
        dc.w    $3F3F,$FCFC
        dc.w    $0000,$0000
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
sm4_size
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $FFFF,$FFFF
        dc.w    $0F0F,$FCFC
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
        dc.w    $0000,$0000
;
        end
