; part of the io library for the PROGS SMS C library
; started 21 October 1994
;   IOGet               get a byte from the channel
;   IOPut               send a byte to the channel
;   IOGetBytes          get multiple bytes from the channel
;   IOPutBytes          send multiple bytes to the channel
;   IOPutRawBytes       send multiple bytes, without translation if possible
;   IOGetString         get a line (\0 or \n terminated) from the channel
;   IOPutString         send a line to the channel (\n added)
;   IOTest              check for pending operation (input) on file
;   IODone              check if all operations done
;   IOFlush             make sure all write operations on a file are done
;   IOTruncate          truncate the file at the current position
;   IOPositionGet       Get current file position
;   IOPositionSet       Set file position
;   IOPositionMove      move file position
; started adding 17 November 1994
;   _IOHeaderGet        internal routine, get file header
;   _IOHeaderSet        internal routine, set file header
;   _IOMedium           internal routine, get extended medium information
;   _IOMakeDirectory    internal routine, make directory
;   _IORename           internal routine, rename file
;   IOFileLoad          scatter load file
;   IOFileSave          scatter save file
;   IOVersionGet        get file version
;   IOVersionSet        set file version
;   IOUpdataTimeGet     get update date
;   IOUpdataTimeSet     set update date
;   IOBackupTimeGet     get backup date
;   IOBackupTimeSet     set backup date
; written by Joachim Van der Auwera for PROGS
; (c) 1994 by PROGS, Professional & Graphical Software, all rights reserved
; 01.02.1996 IOPutBytes bugs when size==0 or size>32768 fixed
; 17.04.1996 IOPutRawBytes added (support iob.suml if exists)
; 05.05.1996 added support for IOPutStringAddNewLine status
; 09.07.1996 IOPositionSet/Move corrupted d3
        section text
        include win1_inc_qdos_io
        include win1_inc_keys_err
        include win1_inc_keys_k
        include win1_inc_aid_mac
;
        xref    TimeOuts
timeout macro
        move.l  a0,-(sp)
        move.l  a0,d3           ; Channel
        add.w   d3,d3           ; calculate offset
        add.w   d3,d3
        move.l  TimeOuts(pc),a0 ; base of timeout table
        move.w  0(a0,d3.w),d3   ; TimeOut
        move.l  (sp)+,a0
        endm
;
; get char from channel
chid    equ     4               ; .l Channel descritor
retc    equ     8               ; .l char *
_IOGet xsub
        move.l  chid(sp),a0     ; Channel
        regsave d3/a1/a2
        move.l  retc+12(sp),a2  ; place for return
        moveq   #iob.fbyt,d0    ; fetch byte
        timeout                 ; set timeout
        trap    #do.io          ; do it
        move.b  d1,(a2)         ; set result
        subend
;
; send char to channel
char    equ     8               ; .w char
_IOPut xsub
        move.l  chid(sp),a0     ; Channel
        move.w  char(sp),d1     ; character
        regsave d3
        moveq   #iob.sbyt,d0    ; send a byte
        timeout
        trap    #do.io          ; do it
        subend
;
; get multiple bytes from channel
length  equ     8               ; number of bytes to get
place   equ     12              ; place for return
count   equ     16              ; count of bytes received
_IOGetBytes xsub
        moveq   #iob.fmul,d0    ; fetch multiple bytes
        bra.s   bytes
;
; send multiple bytes to channel
_IOPutBytes xsub
        moveq   #iob.smul,d0
bytes   move.l  chid(sp),a0     ; Channel
        move.l  place(sp),a1    ; base of buffer
        regsave d3/d4/d5/d7/a2
        move.l  d0,d7           ; remember command to use
        moveq   #0,d0           ; just in case nothing to send
        move.l  length+20(sp),d4; number of bytes to get/send
        move.l  count+20(sp),a2 ; place for return
        moveq   #0,d5           ; number of bytes I've got/sent
        timeout                 ; get timeout
gbloop  move.l  d4,d2           ; length still to handle
        beq.s   gbdone          ; nothing -> done
        cmpi.l  #$8000,d2       ; split up in pieces that os can cope with
        bmi.s   gbd2ok          ; can do
        move.w  #$7fff,d2       ; as much as possible in one call
gbd2ok  move.l  d7,d0           ; command
        trap    #do.io          ; do it
        andi.l  #$ffff,d1       ; make long - unsigned
        sub.l   d1,d4           ; reduce number of bytes to go
        add.l   d1,d5           ; increase number we have done
        tst.w   d0              ; error ?
        beq.s   gbloop          ; no -> continue
gbdone  move.l  a2,d1           ; is the return wanted ??
        beq.s   gbfini
        move.l  d5,(a2)         ; fill in int *got / int *sent
gbfini  subend
;
; send multiple bytes to channel
_IOPutRawBytes xsub
        move.l  chid(sp),a0     ; Channel
        move.l  place(sp),a1    ; base of buffer
        regsave d3/d4/d5/d7/a2
        moveq   #0,d0           ; just in case nothing to send
        moveq   #iob.suml,d7    ; send multiple UNTRANSLATED bytes
        move.l  length+20(sp),d4; number of bytes to get/send
        move.l  count+20(sp),a2 ; place for return
        moveq   #0,d5           ; number of bytes I've got/sent
        timeout                 ; get timeout
ubloop  move.l  d4,d2           ; length still to handle
        beq.s   ubdone          ; nothing -> done
        cmpi.l  #$8000,d2       ; split up in pieces that os can cope with
        bmi.s   ubd2ok          ; can do
        move.w  #$7fff,d2       ; as much as possible in one call
ubd2ok  move.l  d7,d0           ; command
        trap    #do.io          ; do it
        cmpi.w  #err.nimp,d0    ; iob.suml not known ??
        beq.s   uscr
        cmpi.w  #err.ipar,d0    ; alternative of not known
        bne.s   usc
uscr    moveq   #iob.smul,d7    ; use normal send bytes and hope for the best
        bra.s   ubd2ok
usc     andi.l  #$ffff,d1       ; make long - unsigned
        sub.l   d1,d4           ; reduce number of bytes to go
        add.l   d1,d5           ; increase number we have done
        tst.w   d0              ; error ?
        beq.s   ubloop          ; no -> continue
ubdone  move.l  a2,d1           ; is the return wanted ??
        beq.s   ubfini
        move.l  d5,(a2)         ; fill in int *got / int *sent
ubfini  subend
;
; get a string from channel
buff    equ     8               ; .l buffer
space   equ     12              ; .l max length for string
_IOGetString xsub
        move.l  chid(sp),a0     ; Channel
        regsave d3/d4/a2
        move.l  buff+12(sp),a2  ; buffer
        move.l  space+12(sp),d4 ; max length
        subq.l  #1,d4           ; need space for terminating zero
        timeout                 ; get timeout
gsloop  moveq   #err.bffl,d0    ; buffer full ??
        subq.w  #1,d4           ; one less character to go
        bmi.s   gsdone          ; buffer full
        moveq   #iob.fbyt,d0    ; fetch byte
        trap    #do.io          ; do it
        tst.l   d0              ; error ??
        bne.s   gsdone          ; ... oops
        cmpi.b  #k.nl,d1        ; end of line ??
        beq.s   gsdone          ; -> done
        move.b  d1,(a2)+        ; set byte
        bne.s   gsloop          ; not end of string -> erad next
        subq.l  #1,a2           ; - should be faster than jumping pas next op
gsdone  clr.b   (a2)            ; end of string
gsdone2 cmpi.w  #err.eof,d0
        bne.s   gserr
        cmpa.l  buff+12(sp),a2  ; was this the first byte of the string ??
        beq.s   gserr           ; yes -> return error
        moveq   #0,d0           ; something is read -> no eof yet
gserr   subend
;
; put a string from channel
number  equ     12              ; .l (int *) number of characters sent
_IOPutString xsub
        move.l  chid(sp),a0     ; Channel
        regsave d3/d4/a2
        move.l  buff+12(sp),a2  ; buffer
        timeout                 ; get timeout
        moveq   #0,d4           ; number of chars successfully sent
psloop  move.b  (a2)+,d1        ; next byte
        beq.s   pslast
        moveq   #iob.sbyt,d0    ; send byte
        trap    #do.io          ; do it
        tst.w   d0              ; error ??
        bne.s   psdone
        addq.l  #1,d4
        bra.s   psloop
pslast
        move.l  a0,d1           ; Channel
        add.w   d1,d1           ; calculate offset
        add.w   d1,d1
        move.l  TimeOuts(pc),a2 ; base of timeout table
        tst.b   2(a2,d1.w)      ; should I add a newline ??
        beq.s   psdone
        moveq   #k.nl,d1        ; send newline
        moveq   #iob.sbyt,d0    ; send byte
        trap    #do.io          ; do it
        tst.w   d0              ; error ??
        bne.s   psdone
        addq.l  #1,d4
psdone  move.l  number+12(sp),d1; number of bytes sent
        beq.s   psfini          ; oops, result not wanted
        move.l  d1,a2
        move.l  d4,(a2)         ; fill in
psfini  subend
;
; IOTest, IOFlush, IOTruncate
_IOTest xsub
        moveq   #iob.test,d0    ; test
        bra.s   simple
_IODone xsub
        moveq   #iof.chek,d0    ; flush
        bra.s   simple
_IOFlush xsub
        moveq   #iof.flsh,d0    ; flush
        bra.s   simple
_IOTruncate xsub
        moveq   #iof.trnc,d0    ; flush
simple  move.l  chid(sp),a0     ; Channel
        regsave d3
        timeout
        trap    #do.io          ; do it
        subend
;
; get current position in file
pos     equ     8               ; file position
_IOPositionGet xsub
        move.l  chid(sp),a0     ; Channel
        move.l  pos(sp),d1      ; position
        regsave d3/a2
        move.l  d1,a2           ; pos for return
        moveq   #0,d1           ; same position
        timeout                 ; get timeout
        moveq   #iof.posr,d0    ; position relative
        trap    #do.io          ; do it
        move.l  d1,(a2)         ; return current file position
        subend
;
; set new file position
_IOPositionSet xsub
        moveq   #iof.posa,d0
        bra.s   position
;
; move current file position
_IOPositionMove xsub
        moveq   #iof.posr,d0
position
        regsave d3
        move.l  chid+4(sp),a0   ; Channel
        move.l  pos+4(sp),d1    ; position absolute or relative
        timeout
        trap    #do.io
        subend
;
; get/set file header, get medium info
base    equ     8               ; place for file header
__IOHeaderGet xsub
        moveq   #iof.rhdr,d0    ; read header
        bra.s   hdr             ; do it
__IOHeaderSet xsub
        moveq   #iof.shdr,d0    ; set header
        bra.s   hdr             ; do it
__IOMedium xsub
        moveq   #iof.xinf,d0
hdr     move.l  chid(sp),a0     ; Channel
        move.l  base(sp),a1
        regsave d2/d3
        moveq   #64,d2          ; buffer length for iof.rhdr
        timeout
        trap    #do.io          ; do it
        subend
;
; make directory
__IOMakeDirectory xsub
        move.l  chid(sp),a0     ; Channel
        move.l  d3,-(sp)
        moveq   #0,d1           ; obligatory
        moveq   #-1,d3          ; obligatory timeout
        moveq   #iof.mkdr,d0    ; make directory
        trap    #do.io          ; do it
        move.l  (sp)+,d3
        subend
;
; get file version / update/backup date
_IOVersionGet xsub
        moveq   #iof.vers,d0    ; set or read file version
        bra.s   dget
_IOUpdateTimeGet xsub
        moveq   #iof.date,d0    ; set or read file date
        moveq   #0,d1           ; update date
        bra.s   dget
_IOBackupTimeGet xsub
        moveq   #iof.date,d0    ; set or read file date
        moveq   #2,d1           ; backup date
dget    move.l  chid(sp),a0     ; Channel
        move.l  base(sp),a1     ; base for return value
        regsave d2/d3
        move.b  d1,d2           ; put in correct register
        moveq   #-1,d1          ; read it
        timeout
        trap    #do.io          ; do it
        move.l  d1,(a1)         ; fill in return value
        subend
;
; set file version / update/backup date
value   equ     8               ; .l new value
_IOVersionSet xsub
        moveq   #iof.vers,d0    ; set or read file version
        bra.s   dset
_IOUpdateTimeSet xsub
        moveq   #iof.date,d0    ; set or read file date
        sub.l   a1,a1           ; update date
        bra.s   dset
_IOBackupTimeSet xsub
        moveq   #iof.date,d0    ; set or read file date
        sub.l   a1,a1           ; backup date
        addq.l  #2,a1
dset    move.l  chid(sp),a0     ; Channel
        move.l  value(sp),d1    ; base for return value
        regsave d2/d3
        move.w  a1,d2           ; put in correct register
        timeout
        trap    #do.io          ; do it
        subend
;
; rename file
__IORename xsub
        move.l  chid(sp),a0     ; Channel
        move.l  base(sp),a1     ; new name
        regsave d3/a2
        getssp  82              ; need some stack space for filename
        lea     2(sp),a2        ; place to copy string to
        moveq   #79,d2          ; max length of string
        moveq   #-1,d0          ; counter of string length
copystr addq.w  #1,d0           ; string is one longer
        move.b  (a1)+,(a2)+     ; copy another byte
        dbeq    d2,copystr
        move.w  d0,(sp)         ; set length of string
        move.l  sp,a1           ; base of string with new file name
        timeout
        moveq   #iof.rnam,d0    ; rename file
        trap    #do.io          ; do it
        subend
;
; scatter load/save file
_IOFileLoad xsub
        moveq   #iof.load,d0    ; (scatter) load file
        bra.s   scat
_IOFileSave xsub
        moveq   #iof.save,d0    ; (scatter) save file
scat    move.l  chid(sp),a0     ; Channel
        move.l  place(sp),a1    ; base address
        regsave d2/d3
        move.l  8+length(sp),d2 ; length
        timeout
        trap    #do.io          ; do it
        subend
;
        end
