; part of the memory library for the PROGS SMS C library
;   MEMAllocate         allocate some memory
;   MEMAllocateFor      allocate some memory for someone else
;   MEMRelease          release some allocated memory
;   MEMWhenZapped       set routine to call/ byte to set on release
;   MEMFree             determine total amount of free memory
;   MEMLargest          determine largest area of free memory
; written by Joachim Van der Auwera for PROGS
; (c) 1994 by PROGS, Professional & Graphical Software, all rights reserved
; started 12 October 1994
        section text
        include win1_inc_qdos_sms
        include win1_inc_keys_qlv
        include win1_inc_keys_sys
        include win1_inc_aid_mac
;
; routines to allocate some memory
length  equ     4               ; .l length
jobid   equ     8               ; .l SMSJob id
retbase equ     12              ; .l place for return
ptrlen  equ     4               ;    size of a pointer (.l)

_MEMAllocate xsub
        moveq   #sms.myjb,d0    ; job to allocate the memory for
        move.l  retbase-4(sp),a0; place for return value
        bra.s   allocate

_MEMAllocateFor xsub
        move.l  jobid(sp),d0    ; job to allocate the memory for
        move.l  retbase(sp),a0  ; place for return value
allocate
        move.l  length(sp),d1   ; size required
        regsave d2-d4/a2/a3/a4
        move.l  a0,a4           ; keep place for return
        move.w  sr,d2           ; are we in supervisor mode ??
        btst    #$d,d2          ; test the supervisor status bit
        beq.s   allocuser

        move.l  d0,d4           ; keep jobid for a sec

        move.l  d1,-(sp)        ; keep length of area needed
        moveq   #sms.info,d0    ; get jobid and sysvar base
        trap    #do.sms2        ; do it

        cmpi.l  #-1,d4          ; was jobid known ??
        bne.s   gotjobid
        move.l  d1,d4           ; keep my jobid
gotjobid
        move.l  (sp)+,d1        ; length of area needed
        move.l  a6,-(sp)        ; keep a6
        move.l  a0,a6           ; set address of sysvars

        moveq   #ptrlen-memlen,d0
        add.l   d0,d1           ; add header length
        move.w  mem.achp,a0     ; atomic allocate memory
        jsr     (a0)            ; do it

        move.l  (sp)+,a6        ; restore a6
        tst.l   d0              ; error ?
        bne.s   oopsall         ; yes -> ..
        move.l  d1,(a0)+        ; set length
        clr.l   (a0)+           ; no driver just yet
        move.l  d4,(a0)+        ; set owner
        clr.l   (a0)+           ; no ZapAddress just yet
        bra.s   almost          ; finish up
allocuser
        move.l  d0,d2           ; put jobid where it should be
        moveq   #sms.achp,d0    ; allocate some memory
        addq.l  #ptrlen,d1      ; some extra space for ZapRoutine pointer
        trap    #do.sms2        ; do it
        tst.l   d0              ; error ?
        bne.s   oopsall         ; yes -> ooops ..
almost  lea     ZapDriver(pc),a1; set driver to call on release !
        tst.l   12(a1)          ; is driver already usable ??
        bne.s   HandlerOK       ; yes -> ..
        lea     ZapHandler(pc),a2
        move.l  a2,12(a1)       ; set address of handler routine
HandlerOK
        move.l  a1,memdrv(a0)   ; set driver
        clr.l   (a0)+           ; no ZapRoutine just yet
        move.l  a0,(a4)         ; base of requested memory
oopsall subend
;
; release memory
; gentle release, the ZapRoutine and ZapAddress also have to be considered !
; as the OS only does this for a force release
base    equ     4               ; .l base of memory to release
memlen  equ     -16             ; .l offset from real base to length of area
memdrv  equ     -12             ; .l offset from real base to driver
memown  equ     -8              ; .l offset from real base to owner
membyt  equ     -4              ; .l offset from real base to ZapAddress
_MEMRelease xsub
        move.l  base(sp),d0     ; base of area
        regsave d2/d3/d4/a2/a3
        beq.s   relfini         ; base is NULL -> nothing to release
        move.l  d0,a0           ; put base in a0

        move.w  sr,d4           ; are we in supervisor mode ??
        btst    #$d,d4          ; test the supervisor status bit
        bne.s   insuper
        super                   ; all has to be done in supervisor
                                ; to prevent an rjob between calling the
                                ; ZapRoutine and the actual release !
insuper
        move.l  #'FREE',d0      ; see if area already released
        cmp.l   memown-ptrlen(a0),d0
        beq.s   reldone         ; this area was already released !!
        move.l  d0,memown-ptrlen(a0)

        move.l  -ptrlen(a0),d0  ; is there a ZapRoutine ??
        beq.s   noZapRoutine    ; no -> don't call it
        move.l  a0,-(sp)        ; pass base of area to ZapRoutine
        move.l  d0,a0
        jsr     (a0)            ; call ZapRoutine
        move.l  (sp)+,a0        ; restore base in a0
noZapRoutine                    ; base of area still in a0
        subq.l  #ptrlen,a0      ; make that real base
        move.l  membyt(a0),d0   ; is there a ZapAddress
        beq.s   noZapAddress
        move.l  d0,a1           ; ZapAddress
        st      (a1)            ; set it
noZapAddress
        move.l  a0,a2           ; keep base of memory
        moveq   #sms.info,d0    ; get jobid and sysvar base
        trap    #do.sms2        ; do it

        move.l  a6,-(sp)        ; keep a6
        move.l  a0,a6           ; set address of sysvars

        lea     memlen(a2),a0   ; get real base as needed for mem.rchp
        move.w  mem.rchp,a1     ; atomic release memory
        jsr     (a1)            ; do it

        move.l  (sp)+,a6        ; restore a6

reldone btst    #$d,d4          ; did we start in supervisor ??
        bne.s   relfini         ; no -> ok
        user
relfini subend
;
; routine to zet the ZapRoutine and ZapAddress of some area of memory
; this does not work for memory allocated with MEMUserAllocate !!
ZapRoutine equ  8               ; ZapRoutine
ZapAddress equ  12              ; ZapAddress
_MEMWhenZapped xsub
        move.l  base(sp),a0     ; base of area of memory
                                ; handle ZapRoutine
        move.l  ZapRoutine(sp),-(a0)
                                ; handle ZapAddress
        move.l  ZapAddress(sp),membyt(a0)
        moveq   #0,d0           ; no error
        subend
;
; specialised driver to handle ZapRoutines on force remove
ZapDriver dc.l  0,0,0,0
ZapHandler
        move.l  a3,-(sp)        ; we have to preserve a3 !!
                                ; make sure area cannot be explicitly released
        move.l  #'FREE',-memown(a0)
        lea     ptrlen-memlen(a0),a0
        move.l  a0,-(sp)        ; parameter for ZapRoutine
        move.l  -ptrlen(a0),d0  ; base of ZapRoutine
        beq.s   nocall          ; if there is one
        move.l  d0,a0           ; need it in an address
        jsr     (a0)            ; call ZapRoutine
nocall  move.l  (sp)+,a0        ; restore a0
        lea     memlen-ptrlen(a0),a0
        move.w  mem.rchp,a1     ; we have to release this block explicitly
        jsr     (a1)            ; do it
        move.l  (sp)+,a3        ; restore a3
        moveq   #0,d0           ; no error occured !!
        rts                     ; done
;
; get size of largest continous area that can be allocated
result  equ     4               ; .l place to fill in result
_MEMLargest xsub
;
; get total amount of free memory in the system
; this amount gives no gurantees about being able to allocate memory
; as fragmentation can prevent this
; THESE ROUTINES ARE CURRENTLY NOT FULLY FUNCTIONING UNTIL I GET A BETTER
; UNDERSTANDING OF THE SMS MECHANISMS, SO I CAN IMPLEMENT THEM FULLY
_MEMFree xsub
        regsave d2/d3/a2/a3
        moveq   #sms.frtp,d0    ; find largest free space in TP
        trap    #do.sms2        ; do it
        move.l  16+result(sp),a0; place for result
        move.l  d1,(a0)         ; return value
        subend
*
        end
