/*
	Standard library file for PROGS SMS C library

	string handling library
	STRLength
	STRCopySafe
	STRAppendSafe
	STRSameCI
	STRCompareCD
	STRCompareCI
	STRUpper
	STRLower
	STRSearchCD
	STRSearchCI
	STRBreak

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, started 2 November 1994
*/

#include "str_h"
#include "ctype_h"

Error STRLength(char *string, int *length)
{
	*length=strlen(string);
	return ERR_OK;
}

Error STRCopySafe(char *from, char *to, int buffsize)
{
	for (buffsize-- ; buffsize>0 && *from ; buffsize--) *to++=*from++;
	*to='\0';
	if (*from!='\0') return ERR_BFFL;
				else return ERR_OK;
}

Error STRAppendSafe(char *string, char *to, int buffsize)
{
	int len=strlen(to);
	buffsize-=len+1;
	to+=len;
	for ( ; buffsize>0 && *string ; buffsize--) *to++=*string++;
	*to='\0';
	if (*string!='\0') return ERR_BFFL;
				  else return ERR_OK;
}

int STRSameCI(char *string1, char *string2)
{
	char c;
	while (c=*string1++)
	{
		if (toupper(c)!=toupper(*string2++))
		{
			return FALSE;
		}
	}
	if (*string2) return FALSE;
	return TRUE;
}

Error STRCompareCD(char *string, char *with, int *result)
{
	char c,w;

	*result=0;					/* assume equality */
	while (c=*string++)
	{
		w=*with++;
		if (c<w)
		{
			*result=-1;
			return ERR_OK;
		}
		if (c>w)
		{
			*result=1;
			return ERR_OK;
		}
	}
	if (*with)
	{
		*result=-1;
	}
	return ERR_OK;
}

Error STRCompareCI(char *string, char *with, int *result)
{
	char c,w;

	*result=0;					/* assume equality */
	while (c=toupper(*string++))
	{
		w=toupper(*with++);
		if (c<w)
		{
			*result=-1;
			return ERR_OK;
		}
		if (c>w)
		{
			*result=1;
			return ERR_OK;
		}
	}
	if (*with)
	{
		*result=-1;
	}
	return ERR_OK;
}

Error STRUpper(char *string)
{
	char *str=string;
	while (*str++=toupper(*str));
	return ERR_OK;
}

Error STRLower(char *string)
{
	char *str=string;
	while (*str++=tolower(*str));
	return ERR_OK;
}

Error STRSearchCD(char *string, char *in, char **at)
{
	char c, *t, *i;
	char s=*string;

	while(c=*in++)
	{
		if (c==s)
		{
			t=in; i=string+1;
			while (*i && *t++==*i) i++;
			if (!*i)
			{
				/* we reached the end of the string to search */
				/* therefore, we have found it */
				*at=in-1;
				return ERR_OK;
			}
		}
	}
	return ERR_ITNF;
}

Error STRSearchCI(char *string, char *in, char **at)
{
	char c, *t, *i;
	char s=toupper(*string);

	while(c=*in++)
	{
		if (toupper(c)==s)
		{
			t=in; i=string+1;
			while (*i && toupper(*t++)==toupper(*i)) i++;
			if (!*i)
			{
				/* we reached the end of the string to search */
				/* therefore, we have found it */
				*at=in-1;
				return ERR_OK;
			}
		}
	}
	return ERR_ITNF;
}

Error STRBreak(char *string, char *(*list[]))
{
	int length=strlen(string);
	int parts;
	char *strbase, *temp=string;
	char **currptr;
	Error err;

	/* for starters, I need to know how much char pointers I will need */
	/* to avoid having to do the same work twice, I estimate this number */
	/* by counting the number of whitespace runs */
	parts=1;				/* no parts encountered just yet, but need 1 more */
	while (*temp)
	{
		while (isspace(*temp)) temp++;
		if (*temp && !isspace(*temp))
		{
			parts++;		/* we are at the start of another part */
			temp++; 		/* we already tested the current character */
			while (*temp && !isspace(*temp)) temp++;
		}
	}

	/* allocate some space for the pointers and substrings */
	err=MEMAllocate(even(length)+parts*sizeof(char *),list);
	if (err) return err;	/* oops, out of memory */
	currptr=*list;
	strbase=(char *)currptr+parts*sizeof(char *);
	STRCopy(string,strbase);

	/* now divide the string into parts, put the start of each substring */
	/* in *currptr++, and overwrite the strbase string to add eos markers */
	temp=strbase;
	while (*temp)
	{
		while (isspace(*temp)) temp++;
		if (*temp && !isspace(*temp))
		{
			if (*temp=='"')
			{
				temp++; strbase=temp;
				*currptr++=strbase;
				while(TRUE)
				{
					while(*temp && *temp!='"') *strbase++=*temp++;
					if (*temp=='"' && *(temp+1)=='"')
					{
						temp++;
						*strbase++=*temp++;
					}
					else
					{
						*strbase='\0';
						temp++;
						break;
					}
				}
			}
			else
			{
				/* we are at the start of a part */
				*currptr++=temp;
				temp++; 			/* this one is not a space */
				while (*temp && !isspace(*temp)) temp++;
				if (*temp) *temp++='\0';
			}
		}
	}
	*currptr=NULL;
	return ERR_OK;
}
