/*
	Standard library file for PROGS SMS C library

	time handling library
	TIMESplit & TIMEBuild

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, 25 November 1994
*/

#include "time_h"

#define BASEYEAR	1961			/* base year for Time */
static const int sofar_month[]={0,31,59,90,120,151,181,212,243,273,304,334,365};

Error TIMESplit(Time time, SplitTime *res)
{
	int year, month;

	res->seconds=time%60;			/* number of seconds */
	time/=60;
	res->minutes=time%60;			/* number of minutes */
	time/=60;
	res->hour=time%24;				/* number of hours */
	time/=24;						/* time is now in days */
	for(year=0;(year*365+year/4)<time;year++);
	year--;
	res->year=year+BASEYEAR;		/* the year in which it happened */
	time-=year*365+year/4;			/* number of days in the year */
	time++; 						/* start counting from 1 */
	if (time<=31)
	{
		res->day=time;
		res->month=1;
	}
	else if (time<=59)
	{
		res->day=time-31;
		res->month=2;
	}
	else
	{
		if (year%4==3)				/* we have a leap year !! */
		{
			if (time==60)
			{
				res->day=time-31;
				res->month=2;
			}
			else
				time--;
		}
		for(month=3;sofar_month[month]<time;month++);
		res->month=month;
		res->day=time-sofar_month[month-1];
	}
	return ERR_OK;
}

Error TIMEBuild(SplitTime *time, Time *res)
{
	int year,month;
	int sofar;

	year=time->year-BASEYEAR;
	if (year<0) return ERR_ORNG;
	sofar=year*365+year/4;			/* number of days so far */
	month=time->month;
	if (month<1 || month>12) return ERR_ORNG;
	if (year%4==3 && month>2)
		sofar++;					/* leap year, add feb 29 */
	sofar+=sofar_month[month-1]+time->day-1;
	sofar*=24;						/* make that the number of hours */
	sofar+=time->hour;
	sofar*=60;						/* make that the number of minutes */
	sofar+=time->minutes;
	*res=sofar*60+time->seconds;

	return ERR_OK;
}
