; WIN library for the PROGS SMS C library
;   WINRedefine         set new window border, size and position
;   WINWindow           set new window size and position, inside outline !
;   WINBorder           set border width and colour, reduces window size !
;   WINPaper            set paper colour
;   WINStrip            set strip colour
;   WINInk              set ink colour
;   WINUnderLine        set underline status
;   WINOverWrite        set overwrite status
;   WINScroll           scroll window (vertical scroll)
;   WINScrollTop        scroll top of window
;   WINScrollBottom     scroll bottom of window
;   WINPan              pan window (horizontal scroll)
;   WINPanLine          pan cursor line
;   WINPanRight         pan right hand end of cursor line
;   WINClear            clear window
;   WINClearTop         clear top of window
;   WINClearBottom      clear bottom of window
;   WINClearLine        clear cursor line
;   WINClearRight       clear right hand end of cursor line
;   WINCursorEnable     make a cursor visible
;   WINCursorDisable    hide cursor
;   WINPendingNewLine   do a pending newline
;   WINAtCursor         position cursor at cursor coordinates
;   WINAtColumn         position cursor, column only
;   WINNewLine          move cursor to next line
;   WINColumnPrev       move cursor to previous column
;   WINColumnNext       move cursor to next column
;   WINRowPrev          move cursor to previous row
;   WINRowNext          move cursor to next row
;   WINAtPixel          position cursor at pixel position
;   WINInfoCursor       get cursor position of cursor and window size
;   WINInfoPixel        get pixel position of cursor and window size
;   WINCursorSize       set size of cursor (or font size)
;   WINLimits           get window limits, outline should fall in these limits
;   WINOutline          set outline and shadow for window
;   WinAreaSave         save part of window in buffer
;   WINAreaRestore      restore part of window from buffer
;   WINPointerPositionSet  set pointer position
;   WINPointerPositionGet  get pointer position
;   WINPointerLimits    set pointer limits while dragging
;   WINBlock            draw a rectangular block in a window
;   WINSubWindows       set subwindows for a window
;   WINMove             move a window to given position
;   WINSprite           draw a sprite in a window
;   WINMode             get assumed mode for WINOpen/WINOpenFull
;   WINEventSend        send an event to a job
;   WINEventWait        wait for an event
; written by Joachim Van der Auwera for PROGS
; (c) 1994 by PROGS, Professional & Graphical Software, all rights reserved
; started 1 December 1994
; 22.10.1995 WINEdetString removed, WINMode added
; 02.02.1996 WINEventSend and WINEventWait added
; 07.05.1996 all calls with only Window as parameter did not read id properly
; 28.07.1996 WINEventWait clears events when error occured
        section text
        include win1_inc_qdos_io
        include win1_inc_qdos_sms
        include win1_inc_keys_err
        include win1_inc_ptr_keys
        include win1_inc_aid_mac
;
        xref    TimeOuts
timeout macro
        move.l  a0,-(sp)
        move.l  a0,d3           ; Channel
        add.w   d3,d3           ; calculate offset
        add.w   d3,d3
        move.l  TimeOuts(pc),a0 ; base of timeout table
        move.w  0(a0,d3.w),d3   ; TimeOut
        move.l  (sp)+,a0
        endm
;
; routines to get the current system network node number
window  equ     4               ; window channel id
nopar   macro   name,code
_[name] xsub
        moveq   #[code],d0
        bra.s   nopar
        endm

        nopar   WINClearTop,        iow.clrt
        nopar   WINClearBottom,     iow.clrb
        nopar   WINClearLine,       iow.clrl
        nopar   WINClearRight,      iow.clrr
        nopar   WINCursorEnable,    iow.ecur
        nopar   WINCursorDisable,   iow.dcur
        nopar   WINPendingNewLine,  iow.donl
        nopar   WINNewLine,         iow.newl
        nopar   WINColumnPrev,      iow.pcol
        nopar   WINColumnNext,      iow.ncol
        nopar   WINRowPrev,         iow.prow
        nopar   WINRowNext,         iow.nrow
_WINClear xsub
        moveq   #iow.clra,d0
nopar   move.l  window(sp),a0
        move.l  d3,-(sp)
        timeout
        trap    #do.io
        move.l  (sp)+,d3
        subend
;
; fill a rectangular block
color   equ     8               ; colour to use
bdefn   equ     10              ; coordindate list
_WINBlock xsub
        moveq   #iow.blok,d0    ; fill rectangular block
        move.w  color(sp),d1    ; colour
        lea     bdefn(sp),a1    ; block definition
        move.l  window(sp),a0   ; window channel id
        regsave d3
        timeout                 ; fill in timeout
        trap    #do.io          ; do it
        subend
;
; set border width and colour
bwidth  equ     8               ; border width
bcolor  equ     10              ; border colour
wdefn   equ     12              ; window definition
BSP     equ     8
_WINBorder xsub
        moveq   #iow.defb,d0
        bra.s   redf
_WINRedefine xsub
        moveq   #iow.defw,d0
redf    regsave d2/d3
        move.l  BSP+window(sp),a0
        move.w  BSP+bwidth(sp),d2
        move.w  BSP+bcolor(sp),d1
        lea     BSP+wdefn(sp),a1
        timeout
        trap    #do.io
        subend
;
; redefine window size and origin
wwdefn  equ     8               ; window definition
WSP     equ     8
_WINWindow xsub
        regsave d2/d3
        moveq   #iow.defw,d0
        timeout
        move.l  WSP+window(sp),a0
        lea     WSP+wwdefn(sp),a1
        moveq   #0,d2
        trap    #do.io
        subend
;
; set paper/strip/ink colour
value   equ     8
onepar  macro   name,code
_[name] xsub
        moveq   #[code],d0
        bra.s   onepar
        endm

        onepar  WINOverWrite,       iow.sova
        onepar  WINPaper,           iow.spap
        onepar  WINStrip,           iow.sstr
        onepar  WINInk,             iow.sink
        onepar  WINAtColumn,        iow.scol
        onepar  WINScroll,          iow.scra
        onepar  WINScrollTop,       iow.scrt
        onepar  WINScrollBottom,    iow.scrb
        onepar  WINPan,             iow.pana
        onepar  WINPanLine,         iow.panl
        onepar  WINPanRight,        iow.panr
_WINUnderLine xsub
        moveq   #iow.sula,d0
onepar  move.w  value(sp),d1
        move.l  window(sp),a0
        move.l  d3,-(sp)
        timeout
        trap    #do.io
        move.l  (sp)+,d3
        subend
;
; set cursor to pixel position
xpos2   equ     8
ypos2   equ     10
TSP     equ     8
twopar  macro   name,code
_[name] xsub
        moveq   #[code],d0
        bra.s   twopar
        endm

        twopar  WINCursorSize,      iow.ssiz
        twopar  WINAtCursor,        iow.scur
_WINAtPixel xsub
        moveq   #iow.spix,d0
twopar  regsave d2/d3
        move.l  TSP+window(sp),a0
        move.w  TSP+xpos2(sp),d1
        move.w  TSP+ypos2(sp),d2
        timeout
        trap    #do.io
        subend
;
; get info on cursor position and window size
data    equ     8               ; place for information
_WINLimits xsub
        moveq   #iop.flim,d0
        bra.s   info
_WINInfoCursor xsub
        moveq   #iow.chrq,d0
        bra.s   info
_WINInfoPixel xsub
        moveq   #iow.pixq,d0
info    move.l  window(sp),a0
        move.l  data(sp),a1
        regsave d2/d3
        moveq   #0,d2
        timeout
        trap    #do.io
        subend
;
; save (part of) a window
sainfo  equ     8               ; size/pos of window block
sacoor  equ     12              ; position in save area to put block
saarea  equ     16              ; WindowArea *
ASP     equ     12
_WINAreaSave xsub
        regsave d2/d3/a2
        moveq   #iop.svpw,d0
        moveq   #0,d2
        bra.s   area
_WinAreaRestore xsub
        regsave d2/d3/a2
        moveq   #iop.rspw,d0
        moveq   #1,d2
area    move.l  ASP+window(sp),a0
        move.l  ASP+sainfo(sp),a1
        move.l  ASP+saarea(sp),a2
        move.l  ASP+sacoor(sp),d1
        timeout
        trap    #do.io
        subend
;
; (re)define window outline
osdefn  equ     8               ; shadow widths
owdefn  equ     12              ; window definition
OSP     equ     8
_WINOutline xsub
        regsave d2/d3
        moveq   #iop.outl,d0
        timeout
        move.l  OSP+window(sp),a0
        lea     OSP+owdefn(sp),a1
        move.l  OSP+osdefn(sp),d1
        moveq   #0,d2           ; clear contents
        trap    #do.io
        subend
;
; set pointer position
spxpos  equ     8               ; xpos
spypos  equ     10              ; ypos
spspr   equ     12              ; sprite
PSP     equ     8
_WINSprite xsub
        moveq   #iop.wspt,d0
        bra.s   wspt
_WINPointerPositionSet xsub
        moveq   #iop.sptr,d0
wspt    regsave d2/d3
        move.l  PSP+window(sp),a0
        move.w  PSP+spxpos(sp),d1
        swap    d1
        move.w  PSP+spypos(sp),d1
        move.l  PSP+spspr(sp),a1
        moveq   #0,d2           ; absolute coordinates
        timeout
        trap    #3
        subend
;
; read pointer position
gpxpos  equ    8               ; *xpos
gpypos  equ    12              ; *ypos
GPSP     equ     8
_WINPointerPositionGet xsub
        regsave d2/d3
        move.l  GPSP+window(sp),a0
        moveq   #0,d1
        moveq   #0,d2
        timeout
        moveq   #iop.slnk,d0
        trap    #3
        move.l  GPSP+gpxpos(sp),d1
        beq.s   nosxp
        move.l  d1,a0
        move.w  pt_xpos(a1),(a0)
nosxp
        move.l  GPSP+gpypos(sp),d1
        beq.s   nosyp
        move.l  d1,a0
        move.w  pt_ypos(a1),(a0)
nosyp   subend
;
; (re)define window's subwindows
swlist  equ     8               ; sub-windows list
_WINSubWindows xsub
        move.l  window(sp),a0
        move.l  swlist(sp),a1
        move.l  d3,-(sp)
        moveq   #iop.swdf,d0
        timeout
        trap    #do.io
        move.l  (sp)+,d3
        subend
;
; move window outline
mwpos   equ     8               ; new window position
MWSP    equ     12+8
_WINMove xsub
        regsave d2/d3/a2
        getssp  8
        move.l  sp,a1           ; place for param block
        move.l  MWSP+window(sp),a0
        timeout
        lea     getoutlsiz(pc),a2
        moveq   #iow.xtop,d0
        trap    #do.io
        tst.l   d0
        bne.s   movend
        move.l  sp,a1
        move.l  MWSP+mwpos(sp),4(a1)
        moveq   #iop.outl,d0
        timeout
        moveq   #1,d2           ; keep contents
        trap    #do.io
movend  subend
getoutlsiz
        move.l  sd_xhits(a0),(a1)
        move.l  sd_xouts(a0),d1
        sub.l   sd_xhits(a0),d1
        moveq   #0,d0
        rts
;
mode    equ     4
_WINMode xsub
        move.l  mode(sp),d0
        regsave d2/a3/a4
        move.l  d0,a3
        moveq   #-1,d1
        moveq   #-1,d2
        moveq   #sms.dmod,d0
        trap    #do.sms2
        move.b  d1,(a3)
        subend
;
_WINEventSend xsub
        regsave d2/d3
        move.l  8+4(sp),d1
        move.b  8+8(sp),d2
        moveq   #$3a,d0
        trap    #do.sms2
        subend
_WINEventWait xsub
        regsave d2/d3
        move.w  8+4(sp),d3
        move.l  8+6(sp),a0
        move.b  (a0),d2         ; events to wait for
        moveq   #$3b,d0
        trap    #do.sms2
        tst.l   d0              ; in case of error ...
        beq.s   evok            ; none -> ok
        moveq   #0,d2           ; no events occured when error
evok    move.l  8+6(sp),a0      ; a0 is corrupted !! (oops, not documented)
        clr.l   (a0)
        move.b  d2,(a0)         ; events which occured
        subend
;
_WINPointerLimits xsub
        regsave d2/d3
        move.l  4+8(sp),a0      ; Window id
        move.l  08+8(sp),d1     ; origin
        move.l  12+8(sp),d2     ; end coordinates
        moveq   #iop.splm,d0
        trap    #do.io
        subend
;
        end
