 !  C68 Floating Point error routines
 !
 !  If return made from the raise() system call, then an
 !  appropriate value is passed back in d0 and d1.
 !
 !  The values involved are such that 'float' routines 
 !  can simply take the d0 value
 !
 !-----------------------------------------------------------------------------
 !  #1  First version (based on ideas from Michael Mueller)	djw-  09/93
 !  #2  Corrected problem with missing # on .setmaxmin            11/93
 !  #3  Corrected another problem with missing # on .setmaxmin    12/93 
 !-----------------------------------------------------------------------------

	.sect .text

	.define __huge_val
	.define __infinity
	.define __NaN
	.define __SNaN

	.define .overflow
	.define .underflow
	.define .divzero
	.define .setmaxmin

SIGFPE	=	8
EDOM	=	33
ERANGE	=	34

 ! This is the representation of key floating point values.
 !
 ! It is done this way, because it is quite likely that  rounding errors
 ! in the compiler might mean the bit pattern is not exactly represented
 ! if simply done via a #define statement.


! Maximum Floating Point Number (can be signed)
__huge_val:
	.data4	0x7fefffff,0xffffffff

! Infinity  (can be signed)
__infinity:
	.data4	0x7ff00000,0x00000000

! Not-a-Number
__NaN:
	.data4	0x7fffffff,0xffffffff

! Signalling Not-a-Number
__SNaN:
	.data4	0xffffffff,0xffffffff


! Overflow has occurred, return HUGE_VAL

.overflow:
	move.l	__huge_val(pc),d0	! result = HUGE_VAL in case of return
	move.l	__huge_val+4(pc),d1
	rts

! Underflow has occurred, return 0

.underflow:
	move.l	#0,d0			! result = 0 in case of return
	move.l	#0,d1
	rts

! Divide by Zero, return infinity

.divzero:
	move.l	__infinity(pc),d0	! result = infinity in case of return
	move.l	__infinity+4(pc),d1
	rts

! Set LONG_MAX or LONG_MIN
! This is used to set LONG_MAX or LONG_MIN according to the sign of the
! floating point number pointed to by 8(A0)
! (called by sftol() and dftol() routines)

.setmaxmin:
	move.l  8(sp),a0        ! address of value
	move.b  (a0),d1         ! get value
	and.b	#0x80,d1		! isolate sign
	bne	setmin		! ... jump if negative
	move.l	#0x7fffffff,d0	! set to LONG_MAX
	rts
setmin:	move.l	#0x80000000,d0	! set to LONG_MIN
	rts
