#ifndef _SYSLIB_SMS_H
#define _SYSLIB_SMS_H

/*
	Standard header file for PROGS SMS C library

	SMS definitions

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, 6 October 1994
*/

#include <err.h>

#ifndef NULL
#define NULL 0
#endif

#ifndef NUL
#define NUL 0
#endif

#ifndef TRUE
#define TRUE -1
/* strictly speaking, C interprets any non-zero value as true, so -1 will do */
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef max
#define max(a,b) ( (a)>(b) ? (a) : (b) )
#endif

#ifndef min
#define min(a,b) ( (a)<(b) ? (a) : (b) )
#endif

#ifndef newmax
#define newmax(old,chk) do { if ((chk)>(old)) old=chk; } while (0)
#endif

#ifndef newmin
#define newmin(old,chk) do { if ((chk)<(old)) old=chk; } while (0)
#endif

#ifndef abs
#define abs(a) ( ((a)<0) ? (-(a)) : (a) )
#endif

#ifndef even
/* macro to make an integer even, always larger or equal to original */
#define even(x) (((x)+1)&-2)
#endif

#ifndef offsetof
#define offsetof(type,identifier) ((Size)&((type *)0)->identifier)
#endif

#define INFINIT (Size)0x7fffffff
typedef int Size;

#define MYSELF (Job)-1
#define SYSTEM (Job)0
typedef long Job;					/* SMS Job Identifier type */

typedef unsigned short BaudRate;
typedef unsigned short Duration;
typedef unsigned char Pitch;

#define RANDOM_MAX 2147483647		/* maximum result for SMSRandom */
#define RANDOM_INIT 1				/* initial suggested value for seed */

Error SMSVersion(char *version);
Error SMSBaud(BaudRate baudrate);
Error SMSBeep(Duration duration, Pitch pitch);
Error SMSRandom(int *result);
Error SMSRandomSeed(int newseed);
void SMSBreakSoft();
void SMSBreakHard();

#endif _SYSLIB_SMS_H
