/*
	test program for PROGS SMS C library

	program to create the files library file for syslib (and similar DLL's)
	for comments, see dll_syslib_txt
	One parameter, input file with names ( XXX_dll, on data device)
	generates libXXX_a and dllXXX_s on data device

	written by Joachim Van der Auwera for PROGS
	(c) 1995 by PROGS, Professional & Graphical Software
*/

#include "sms_h"
#include "io_h"
#include "str_h"
#include "job_h"
#include "win_h"

#define tell(x) WINPutString(win,x)
#define put(x) IOPutString(out,x,NULL)

#define TEMP  "tmp_s"
#define TEMPO "tmp_o"
char str1[]={TEMP};
char str2[]={"-o"};
char str3[]={">&NULL"};
char str4[]={TEMPO};
char *as_parm[]={str1,str2,str4,str3};

static WindowDescriptor report_window={ 350,50,85,80,6,6,0,1,2,7 };

void report(char *oops)
{
	Error err;
	Window win;
	char chr;

	SMSBeep(2000,20);
	WINOpenFull(&report_window,&win);
	tell(oops);
	tell("press a key");
	WINGet(win,&chr);
	WINClose(win);
}

Error append(Channel out)
{
	Error err;
	Channel in;
	Size len;
	char *base, line[80];

	err=IOOpenPath(TEMPO,OPEN_OLD,PATH_DATAPROG,&in);
	if (err)
	{
		STRFormatPutSafe(line,80,"can't open file %s",line);
		report(line);
		return err;
	}
	err=IOLength(in,&len);
	if (err) { report("can't determine length of file"); return err; }
	err=MEMAllocate(len,&base);
	if (err) { report("can't allocate the memory"); return err; }
	err=IOFileLoad(in,len,base);
	if (err) { report("can't load file"); return err; }
	err=IOPutBytes(out,len,base,NULL);
	if (err) { report("can't append file"); return err; }
	MEMRelease(base);
	IOClose(in);
}


Error init(char *name)
{
	Error err;
	Channel in, out, arr, lib;
	char line[160], input[80], basename[80];
	short count=0;

	/* open the input file */

	STRCopy(name,line);
	STRAppend("_dll",line);
	err=IOOpenPath(line,OPEN_READ,PATH_DATA,&in);
	if (err)
	{
		STRFormatPut(input,"can't open input file %s",line);
		report(input);
		return err;
	}

	/* open output file */

	STRCopy("lib",line);
	STRAppend(name,line);
	STRAppend("_a",line);
	err=IOOpenPath(line,OPEN_OVER,PATH_DATA,&lib);
	if (err)
	{
		STRFormatPutSafe(line,160,"can't open library file %s",line);
		report(line);
		return err;
	}

	/* generate the start of the library */

	err=IOOpenPath(TEMP,OPEN_OVER,PATH_DATA,&out);
	if (err) { report("can't open temporary file"); return err; }

	/* fill in name of routine to pull in the library */
	IOGetString(in,basename,80);
	STRFormatPut(line,".text\n dc.l endof%s\n.globl %s\n%s:\n rts",name,basename,basename); put(line);

	STRFormatPut(line,"\t.text\n\t.even\n\t.ascii \"<<QD%s\"\n\tdc.l\t%s",
								"LL>>",name); put(line);

	/* fill in extension name */
	IOGetString(in,input,80);
	while (strlen(input)<4) STRAppend (" ",input);
	if (strlen(input)!=4) {report("extension should be 4 chars long"); return err; }
	STRFormatPut(line,"\t.ascii \"%s\"",input); put(line);

	/* fill in thingname */
	IOGetString(in,input,80);
	STRFormatPut(line,"\tdc.w %d\n\t.ascii \"%s\"",strlen(input),input); put(line);

	/* fill in version */
	IOGetString(in,input,80);
	if (*input)
	{
		STRFormatPut(line,"\t.ascii \" %s\"",input); put(line);
	}

	/* and the end of the file */
	STRFormatPut(line,"\n .bss\n%s:\n dc.w 0",name); put(line);

	/* compile this file to be the start of the library */
	IOClose(out);
	
	err=JOBLoad("as68",PATH_PROGDATA,32,MYSELF,TRUE,4,as_parm,0,NULL,NULL);
	if (err) { report("can't compile start of library"); return err; };
	if (err=append(lib)) return err;

	/* open file for array */

	STRCopy("dll",line);
	STRAppend(name,line);
	STRAppend("_s",line);
	err=IOOpenPath(line,OPEN_OVER,PATH_DATA,&arr);
	if (err) { report("can't open temporary file"); return err; }
	STRFormatPut(line," .globl _DLLARRAY%s\n_DLLARRAY%s:",name,name); IOPutString(arr,line,NULL);

	/* generate the files for each routine (or whatever) */

	while (IOGetString(in,input,80)!=ERR_EOF)
	{
		static char underscore[2]="?";
		static char data[2]="?";
		short size;

		count++;
		if (input[0]=='%' || input[0]=='\0')
		{
			IOPutString(arr," dc.l 0",NULL);
			continue;
		}
		if (input[0]!='.') underscore[0]='_';
					  else underscore[0]='\0';
		if (input[strlen(input)-1]=='@')
		{
			input[strlen(input)-1]='\0';
			data[0]='4';
			size=0x8000;
		}
		else
		{
			data[0]='6';
			size=0x4000;
		}

		err=IOOpenPath(TEMP,OPEN_OVER,PATH_DATA,&out);
		if (err) { report("can't open temporary file"); return err; }

		STRFormatPut(line,".text\n.even\n dc.w 0x%x\n.bss\n.globl %s%s\n%s%s:\n ds.b %s",
			(unsigned int)(count+size),underscore,input,underscore,input,data); put(line);
		STRFormatPut(line," dc.l %s%s",underscore,input); IOPutString(arr,line,NULL);

		IOClose(out);
		err=JOBLoad("as68",PATH_PROGDATA,32,MYSELF,TRUE,4,as_parm,0,NULL,NULL);
		if (err) { report("can't compile part of library"); return err; };
		if (err=append(lib)) return err;
	}

	/* generate terminating file */

	err=IOOpenPath(TEMP,OPEN_OVER,PATH_DATA,&out);
	if (err) { report("can't open temporary file"); return err; }

	STRFormatPut(line,".text\n.globl endof%s\nendof%s:\n dc.w 0",name,name); put(line);

	IOClose(out);
	err=JOBLoad("as68",PATH_PROGDATA,32,MYSELF,TRUE,4,as_parm,0,NULL,NULL);
	if (err) { report("can't compile part of library"); return err; };
	if (err=append(lib)) return err;

	/* end file with array */

	STRFormatPut(line," .globl _COUNT%s\n_COUNT%s:\n dc.w %d",name,name,count); IOPutString(arr,line,NULL);

	IOClose(lib);
	IOClose(arr);
	IOClose(in);
}
