/*
	test program for PROGS SMS C library

	Transform output file of inker into an executable with proper header.
	This also increases the dataspace by 4k !
	The file is searched on the data default only.
	Parameters : file to transform and program name.

	written by Joachim Van der Auwera for PROGS
	(c) 1994 by PROGS, Professional & Graphical Software
*/

#include "sms_h"
#include "io_h"
#include "str_h"
#include "job_h"

#define tell(x) IOPutString(chid,x,&sent)
#define put(ch,x) IOPutString(ch,x,&sent)

#define HEADLEN 10

void report(char *oops)
{
	Error err;
	Channel chid;
	int sent; char chr;

	IOOpen("con",OPEN_OVER,&chid);
	tell(oops);
	tell("press a key");
	IOGet(chid,&chr);
	IOClose(chid);
}

Error init(char *param)
{
	Error err;
	Channel out;
	char line[80];
	Size orglen, len, space;
	char **list, *base;
	int i,sent;

	/* get the command line parameters */
	err=STRBreak(param,&list);
	if (err) return err;
	if (*list==NULL) return ERR_ISYN;

	err=IOOpenPath(*list++,OPEN_OLD,PATH_DATA,&out);
	if (err)
	{
		STRFormatPutSafe(line,80,"can't open file %s",line);
		report(line);
		return err;
	}

	if (*list==NULL) return ERR_ISYN;

	err=IOLength(out, &orglen);
	if (err) { report("can't get length of input file"); return err; }
	err=IODataSpaceGet(out, &space);
	if (err) { report("can't get dataspace of input file"); return err; }
	len=orglen+HEADLEN+JOB_MAXNAME+space;
	err=MEMAllocate(len,&base);
	if (err) { report("can't allocate the memory"); return err; }

	/* figure out amount of dataspace to use */
	space=0;
	if (list[1]) STRFormatGet(list[1],NULL,"%d",&space);
	space+=4096;
	if (space<2048) space=2048;

	/* fill in the header */
	base[0]=0x60; base[1]=HEADLEN-2+JOB_MAXNAME;
	*((int *)(base+2))=space;
	base[6]=0x4a; base[7]=0xfb;
	base[9]=min(strlen(*list),JOB_MAXNAME);
	for (i=HEADLEN ; i<HEADLEN+JOB_MAXNAME-1 ; i++) base[i]=' ';
	MEMCopy(base[9],*list,base+HEADLEN);
	for (i=64 ; i ; i--)
		if (base[i+HEADLEN-1]=='')
			base[i+HEADLEN-1]=' ', base[9]=i-1;

	err=IOFileLoad(out,orglen,base+HEADLEN+JOB_MAXNAME);
	if (err) { report("can't load file"); return err; }

	/* now write the file back */
	err=IOPositionSet(out,0);		/* reset file position */
	if (err) { report("can't reset file"); return err; }
	err=IOPutBytes(out,len,base,&sent);
	if (err) { report("can't append file"); return err; }

	err=IODataSpaceSet(out,space);
	if (err) { report("can't set dataspace value of file"); return err; }
	
	return IOClose(out);
}
